import sys
import itertools
import numpy

######## Simplex implementation
##
# Table format:
# Z|-objective
# -+-----------
# r| co
# h|   ns
# s|      tr

EPSILON = 1e-10
INF = 1e10

def pivot_column(table):
    """Find column used for pivoting (the one with maximum negative objective value)
    """
    objective = table[0, 1:]
    m = numpy.min(objective)
    if m >= -EPSILON:
        return 0
    return 1 + numpy.argmin(objective)

def pivot_row(table, col):
    """Find pivoting row given pivoting column. The one with maximum ratio to rhs
    """
    lhs = table[1:, col]
    rhs = table[1:, 0]
    ratio = [r / l if l > EPSILON else INF for l,r in zip(lhs, rhs)]
    assert min(ratio) < INF
    return 1 + numpy.argmin(ratio)

def do_pivot(table):
    """Make one pivoting step, returns true on success
    """
    c = pivot_column(table)
    if not c:
        return False
    r = pivot_row(table, c)
    x = table[r][c]
    # normalize pivot row
    table[r] /=  x
    # eliminate all other rows
    for t in range(len(table)):
        x = table[t][c]
        if x == 0 or t == r:
            continue
        table[t] -= table[r] * x
    return True

def build_simplex_table(objective, constraints, rhs):
    """Create simplex tableau used by the algorithm.
    We assume that the problem is in its basic form, i.e.
    1. all variables are assumed to be >= 0
    2. all constrains are in less-than-or-equal right-hand-side form
    3. all variables set to 0 is primal feasible solution

    Args:
        objective: objective coefficients for all variables
        constraints: list of constraints, each constraint
            contains coefficients of variables in that constraint
        rhs: list of right hand sides of constraints
    """
    n = len(constraints)
    m = len(objective)
    assert len(constraints) == len(rhs)
    cols = n + m + 1
    rows = n + 1
    data = numpy.zeros((rows, cols))
    for i, x in enumerate(objective):
        data[0][i+1] = -x
    for i, row in enumerate(constraints):
        for j, x in enumerate(row):
            data[i+1][j+1] = x
        data[i+1][0] = rhs[i]
    for i in range(n):
        data[i+1][m+i+1] = 1 # slack variables
    return data

def simplex_solve(objective, constraints, rhs):
    """@see build_simplex_table for arguments
    """
    data = build_simplex_table(objective, constraints, rhs)
    while do_pivot(data):
        pass
    return data[0][0]


################# And here starts the real solution

def read_testcase(fin):
    N, source, sink, L = [int(x) for x in fin.readline().strip().split(" ")]

    capacity = []

    for _ in range(N):
        line = fin.readline().strip().split(" ")
        capacity.append([int(x) for x in line])
    return N, source, sink, L, capacity

def list_testcases(fin):
    cases = int(fin.readline().strip())

    for _ in range(cases):
        fin.readline()
        yield read_testcase(fin)


def solve_testcase(N, source, sink, L, capacity):
    id_gen = itertools.count()
    # flow[u][v][i] holds variable id or None of no variable is needed (there is no edge)
    flow = [[[ id_gen.next() if capacity[u][v] else None for k in range(L)] for v in range(N)] for u in range(N)]
    # total number of variables
    VARS = id_gen.next()

    constraints_coefs = []
    constraints_rhs = []
    zero_variables = [] # holds ids of provably zero variables, for pre-solving optimization

    def add_less_equal_constraint(var_coefs, right):
        """ Add constraint <= rhs.
        Args:
            var_coefs is a list of tuples (variable_id, coeficient)
        """
        constr = numpy.zeros(VARS)
        for var, coef in var_coefs:
            constr[var] += coef
        constraints_coefs.append(constr)
        constraints_rhs.append(right)

    def add_equal_zero_constraint(var_coefs):
        # Equalities are just two inequalities together
        # Note: we do not support other that "equals zero" because otherwise
        # solution all=0 might not be feasible for primal
        add_less_equal_constraint(var_coefs, 0)
        add_less_equal_constraint([(var, -coef) for var,coef in var_coefs], 0)

    #### Ok, now add constraints

    # Capacity constraints
    for u in range(N):
        for v in range(N):
            if not capacity[u][v]:
                continue # skip non-edges
            add_less_equal_constraint(
                [(var, 1) for var in flow[u][v]],
                capacity[u][v]
            )

    # Flow conservation law at each level
    for v in range(N):
        if v == source or v == sink:
            continue # no flow conservation for sink and source
        for i in range(L - 1):
            # use minus sign for incoming
            incoming = [(flow[u][v][i],  -1) for u in range(N) if capacity[u][v]]
            outgoing = [(flow[v][w][i+1], 1) for w in range(N) if capacity[w][v]]
            add_equal_zero_constraint(incoming + outgoing)

    # Flow is generated only at sink at level 0
    for u in range(N):
        if u != source:
            for v in range(N):
                if capacity[u][v]:
                    zero_variables.append(flow[u][v][0])

    # Flow is consumed only by sink at level L-1
    for u in range(N):
        for v in range(N):
            if v != sink:
                if capacity[u][v]:
                    zero_variables.append(flow[u][v][L-1])

    # Avoid self-loop at the source
    for u in range(N):
        if capacity[u][source]:
            for i in range(L):
                zero_variables.append(flow[u][source][i])

    # objective
    objective = numpy.zeros(VARS)
    for u in range(N):
        if capacity[source][u]: # implies also u != source:
            objective[flow[source][u][0]] = 1

    # prune columns of zero variables
    objective = numpy.delete(objective, zero_variables)
    constraints_coefs = [numpy.delete(c, zero_variables) for c in constraints_coefs]

    print 1.0 / 8 * simplex_solve(objective, constraints_coefs, constraints_rhs)


if __name__ == "__main__":
    for params in list_testcases(sys.stdin):
        solve_testcase(*params)

